package cn.iocoder.yudao.module.system.controller.admin.equipment;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.attendancemachine.*;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtRespVO;
import cn.iocoder.yudao.module.system.service.equipment.AttendanceMachineService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


@Tag(name = "管理后台 - 考勤设备")
@RestController
@RequestMapping("/system/attendance-machine")
@Validated
public class AttendanceMachineController {

    @Resource
    private AttendanceMachineService attendanceMachineService;

    @PostMapping("/create")
    @Operation(summary = "创建考勤设备")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:create')")
    public CommonResult<Long> createAttendanceMachine(@Valid @RequestBody AttendanceMachineSaveReqVO createReqVO) {
        return success(attendanceMachineService.createAttendanceMachine(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新考勤设备")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:update')")
    public CommonResult<Boolean> updateAttendanceMachine(@Valid @RequestBody AttendanceMachineSaveReqVO updateReqVO) {

        attendanceMachineService.updateAttendanceMachine(updateReqVO);
        return success(true);
    }

    @PutMapping("/update-password")
    @Operation(summary = "考勤设备密码修改")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:update')")
    public CommonResult<Boolean> updateAttendanceMachine(@Valid @RequestBody AttendanceMachinePasswordVO updateReqVO) {

        attendanceMachineService.updatePassword(updateReqVO);
        return success(true);
    }

    @PutMapping("/update-user")
    @Operation(summary = "考勤设备用户下发")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:update')")
    public CommonResult<Boolean> addUserToAttendanceMachine(@Valid @RequestBody AddUserToAttendanceMachineVO addReqVO) {

        attendanceMachineService.addUserToAttendanceMachine(addReqVO);
        return success(true);
    }

    @GetMapping("/getUserByDeviceNo")
    @Operation(summary = "获得设备已下发的用户列表")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:query')")
    public CommonResult<PageResult<UsersExtRespVO>> getUserByDeviceNo(UserExtPageReqVO pageReqVO) {

        PageResult<UsersExtRespVO> userPage = attendanceMachineService.getUserByDeviceNo(pageReqVO);

        return success(userPage);
    }

    @GetMapping("/get")
    @Operation(summary = "获得考勤设备")
    @Parameter(name = "assetsNo", description = "资产编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:query')")
    public CommonResult<AttendanceMachineRespVO> getAttendanceMachine(@RequestParam("assetsNo") String assetsNo) {

        AttendanceMachineRespVO attendanceMachine = attendanceMachineService.getAttendanceMachine(assetsNo);

        return success(attendanceMachine);
    }

    @GetMapping("/page")
    @Operation(summary = "获得考勤设备分页")
    @PreAuthorize("@ss.hasPermission('system:attendance-machine:query')")
    public CommonResult<PageResult<AttendanceMachineRespVO>> getAttendanceMachinePage(@Valid AttendanceMachinePageReqVO pageReqVO) {

        PageResult<AttendanceMachineRespVO> pageResult = attendanceMachineService.getAttendanceMachinePage(pageReqVO);

        return success(pageResult);
    }
}