package cn.iocoder.yudao.module.system.service.attendance.group;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.Constants;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.group.vo.AttendanceGroupPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.group.vo.AttendanceGroupSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupuser.AttendanceGroupUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.punchrecord.AttendancePunchRecordDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.group.AttendanceGroupMapper;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.groupuser.AttendanceGroupUserMapper;
import cn.iocoder.yudao.module.system.service.attendance.AttendanceService;
import cn.iocoder.yudao.module.system.service.attendance.fixed.AttendanceFixedService;
import cn.iocoder.yudao.module.system.service.attendance.groupshiftitem.AttendanceGroupShiftItemService;
import cn.iocoder.yudao.module.system.service.attendance.punch.dto.AttendanceOnTheDayDTO;
import cn.iocoder.yudao.module.system.service.attendance.punchrecord.AttendancePunchRecordService;
import cn.iocoder.yudao.module.system.service.attendance.scheduling.AttendanceSchedulingService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.GROUP_NOT_EXISTS;

/**
 * 考勤组 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class AttendanceGroupServiceImpl implements AttendanceGroupService {

    @Resource
    private AttendanceGroupMapper groupMapper;
    @Resource
    private AttendancePunchRecordService attendancePunchRecordService;
    @Resource
    private AttendanceGroupMapper attendanceGroupMapper;
    @Resource
    private AttendanceGroupUserMapper attendanceGroupUserMapper;
    @Resource
    private AttendanceGroupShiftItemService attendanceGroupShiftItemService;
    @Resource
    @Lazy // 避免依赖循环
    private AttendanceService attendanceService;
    @Resource
    private AttendanceFixedService attendanceFixedService;
    @Resource
    private AttendanceSchedulingService attendanceSchedulingService;


    @Override
    public Long createGroup(AttendanceGroupSaveReqVO createReqVO) {
        // 插入
        AttendanceGroupDO group = BeanUtils.toBean(createReqVO, AttendanceGroupDO.class);
        groupMapper.insert(group);
        // 返回
        return group.getId();
    }

    @Override
    public void updateGroup(AttendanceGroupSaveReqVO updateReqVO) {
        // 校验存在
        validateGroupExists(updateReqVO.getId());
        // 更新
        AttendanceGroupDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceGroupDO.class);
        groupMapper.updateById(updateObj);
    }

    @Override
    public void deleteGroup(Long id) {
        // 校验存在
        validateGroupExists(id);
        // 删除
        groupMapper.deleteById(id);
    }

    private void validateGroupExists(Long id) {
        if (groupMapper.selectById(id) == null) {
            throw exception(GROUP_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceGroupDO getGroup(Long id) {
        return groupMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceGroupDO> getGroupPage(AttendanceGroupPageReqVO pageReqVO) {
        return groupMapper.selectPage(pageReqVO);
    }

    @Override
    public AttendanceGroupDO getByUserId(Long userId) {
        List<AttendanceGroupDO> dos = groupMapper.getByUserId(userId);
        if (!dos.isEmpty()) {
            return dos.get(0);
        }
        return null;
    }

    @Override
    public void test() {
        // 获取所有考勤组
        List<AttendanceGroupDO> attendanceGroupDOS = attendanceGroupMapper.selectList();
        // -- 根据考勤组ids 获取所有人员
        List<AttendanceGroupUserDO> attendanceGroupUserDOS = attendanceGroupUserMapper.selectList();
        //将attendanceGroupUserDOS对象中的type相同的userId合并成一个数组 type作为key 用户id列表List<Long>作为value
        Map<Long, List<Long>> groupUserMap = attendanceGroupUserDOS.stream().collect(Collectors.groupingBy(AttendanceGroupUserDO::getAttendanceGroupId,
                Collectors.mapping(AttendanceGroupUserDO::getUserId, Collectors.toList())));
        // -- 获取考勤组下考勤规则 - 将将考勤组分组 - 按类型
        LocalDateTime tomorrowLocalDateTime = LocalDateTimeUtil.offset(LocalDateTime.now(), 1, ChronoUnit.DAYS);
        String time = tomorrowLocalDateTime.format(Constants.REPO_DATE_FORMAT);
        // 获取到考勤组 - 班次 key/value 格式
        Map<Long, Long> map = this.getAttendanceGroupShiftIdGroupByGroup(attendanceGroupDOS, tomorrowLocalDateTime);
        Map<Long, AttendanceGroupDO> groupMap = attendanceGroupDOS.stream().collect(Collectors.toMap(AttendanceGroupDO::getId, v -> v));
        // -- 获取班次子表列表
        if (MapUtil.isNotEmpty(map)) {
            List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOList = attendanceGroupShiftItemService.getGroupShiftItemListByShiftIds(new ArrayList<>(map.values()));
            Map<Long, List<AttendanceGroupShiftItemDO>> itemMaps = attendanceGroupShiftItemDOList.stream().collect(Collectors.groupingBy(AttendanceGroupShiftItemDO::getKqAttendanceGroupShiftId));
            List<AttendancePunchRecordDO> attendancePunchRecordDOList = new ArrayList<>();
            for (Map.Entry<Long, Long> entry : map.entrySet()) {
                AttendanceGroupDO attendanceGroupDO = groupMap.get(entry.getKey());
                List<Long> userIds = groupUserMap.get(entry.getKey());
                if (CollectionUtil.isEmpty(userIds)){
                    continue;
                }
                List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOS = itemMaps.get(entry.getValue());
                if (CollectionUtil.isEmpty(attendanceGroupShiftItemDOS)){
                    continue;
                }
                List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = attendanceService.buildAttendanceOnTheDay(attendanceGroupShiftItemDOS);
                for (Long userId : userIds) {
                    for (AttendanceOnTheDayDTO attendanceOnTheDayDTO : attendanceOnTheDayDTOS) {
                        AttendancePunchRecordDO attendancePunchRecordDO = new AttendancePunchRecordDO();
                        attendancePunchRecordDO.setUserId(userId);
                        attendancePunchRecordDO.setAttendanceGroupId(entry.getKey());
                        attendancePunchRecordDO.setAttendanceGroupShiftId(attendanceOnTheDayDTO.getKqAttendanceGroupShiftId());
                        attendancePunchRecordDO.setAttendanceGroupShiftName(attendanceOnTheDayDTO.getKqAttendanceGroupShiftName());
                        attendancePunchRecordDO.setAttendanceGroupShiftItemId(attendanceOnTheDayDTO.getId());
                        attendancePunchRecordDO.setType(attendanceGroupDO.getType());
                        attendancePunchRecordDO.setPunchType(attendanceGroupDO.getPunchType());
                        attendancePunchRecordDO.setWorkType(attendanceOnTheDayDTO.getType());
                        attendancePunchRecordDO.setLevel(attendanceOnTheDayDTO.getLevel());
                        attendancePunchRecordDO.setStatus(AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH);
                        attendancePunchRecordDO.setFieldServiceFlag(Constants.FALSE);
                        attendancePunchRecordDO.setDayTime(time);
                        LocalDateTime shouldPunchTime = LocalDateTime.ofInstant(DateUtils.buildHHmmTime(attendanceOnTheDayDTO.getTime()).toInstant(), ZoneId.systemDefault());
                        attendancePunchRecordDO.setShouldPunchTime(shouldPunchTime);
                        attendancePunchRecordDOList.add(attendancePunchRecordDO);
                        // TODO: 2024/5/24
                    }
                }
            }
            // -- 批量
            attendancePunchRecordService.saveBatch(attendancePunchRecordDOList);
        }
    }


    private Map<Long, Long> getAttendanceGroupShiftIdGroupByGroup(List<AttendanceGroupDO> attendanceGroupDOS, LocalDateTime localDateTime) {
        Map<Long, Long> map = new HashMap<>();
        List<AttendanceGroupDO> fixedList = attendanceGroupDOS.stream().filter(a -> a.getType().equals(1)).collect(Collectors.toList());
        List<AttendanceGroupDO> schedulingList = attendanceGroupDOS.stream().filter(a -> a.getType().equals(2)).collect(Collectors.toList());
        if (!fixedList.isEmpty()) {
            Map<Long, Long> attendanceFixedMap = attendanceFixedService.getGroupToShiftIdMap(fixedList, localDateTime);
            if (ObjectUtil.isNotEmpty(attendanceFixedMap)) {
                map.putAll(attendanceFixedMap);
            }
        }
        if (!schedulingList.isEmpty()) {
            Map<Long, Long> attendanceSchedulingMap = attendanceSchedulingService.getGroupToShiftIdMap(schedulingList, localDateTime);
            if (ObjectUtil.isNotEmpty(attendanceSchedulingMap)) {
                map.putAll(attendanceSchedulingMap);
            }
        }
        return map;
    }

}