package cn.iocoder.yudao.module.system.service.equipment;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.IoUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UserExtImportVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtRespVO;
import cn.iocoder.yudao.module.system.controller.admin.equipment.vo.userExt.UsersExtSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserImportRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.upload.UploadUserFile;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.UsersExtDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.mysql.equipment.UsersExtMapper;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 用户信息拓展 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class UsersExtServiceImpl implements UsersExtService {

    @Resource
    private UsersExtMapper usersExtMapper;

    @Resource
    @Lazy
    private DeptService deptService;

    @Resource
    @Lazy
    private FileApi fileApi;

    @Resource
    private AdminUserService userService;

    @Override
    public Long createUsersExt(UsersExtDO updateDO, MultipartFile file) throws IOException {

        String url = null;
        String name = null;
        String fileName = file.getOriginalFilename();
        if (fileName == null || fileName.isEmpty()) {

            name = updateDO.getUserId() + "_faceImg.jpg";
        }else {

            name = updateDO.getUserId() + "_faceImg." + fileName.split("\\.")[1];
        }
        byte[] content = IoUtil.readBytes(file.getInputStream());

        //校验 当前用户人脸信息是否已存在
        UsersExtDO usersExtDO = usersExtMapper.selectOne(UsersExtDO::getUserId, updateDO.getUserId());
        if (usersExtDO == null) {

            // 同步 把人脸图片插入 BusinessFile中 存储文件
            url = fileApi.createBusinessFile(6L, name, content) ;

            updateDO.setFaceImg(url);
            usersExtMapper.insert(updateDO);

        }else {

            //有人脸图片的情况
            if (usersExtDO.getFaceImg() != null) {

                // 变更business_file的content字段内容
                url = fileApi.updateBusinessFileContent(usersExtDO.getFaceImg(), 6L, name, content);

                //更新人脸图片
                updateDO.setId(usersExtDO.getId());
                updateDO.setFaceImg(url);
                usersExtMapper.updateById(usersExtDO);
            }
        }

        // 返回
        return updateDO.getUserId();
    }

    @Override
    public void updateUsersExt(UsersExtSaveReqVO updateReqVO) {
        // 校验存在
        validateUsersExtExists(updateReqVO.getUserId());
        // 更新
        UsersExtDO updateObj = BeanUtils.toBean(updateReqVO, UsersExtDO.class);

        usersExtMapper.updateById(updateObj);
    }

    @Override
    public void updateListUsersExt(List<UsersExtRespVO> updateDo) {

        // 更新
        List<UsersExtDO> update = BeanUtils.toBean(updateDo, UsersExtDO.class);

        usersExtMapper.updateBatch(update);
    }

    @Override
    public void deleteUserFaceImg(Long userId) throws Exception {

        //校验 当前用户人脸信息是否已存在
        UsersExtDO usersExtDO = usersExtMapper.selectOne(UsersExtDO::getUserId, userId);
        if (usersExtDO != null) {

            //删除 文件存储器中人脸图片
            fileApi.deleteBpmFile(usersExtDO.getFaceImg());

            //更新 清空人脸图片信息
            usersExtMapper.updateFaceImg(usersExtDO.getId());
        }
    }

    private void validateUsersExtExists(Long userId) {

        if (usersExtMapper.selectOne(UsersExtDO::getUserId, userId) == null) {
            throw exception(USERS_EXT_NOT_EXISTS);
        }
    }

    @Override
    public UsersExtDO getUsersExt(Long id) {
        return usersExtMapper.selectById(id);
    }

    @Override
    public PageResult<UsersExtRespVO> getUsersExtPage(UsersExtPageReqVO pageReqVO) {

        if (pageReqVO.getDeptId() != null) {

            List<Long> deptIds = convertList(deptService.getChildDeptList(pageReqVO.getDeptId()), DeptDO::getId);
            deptIds.add(pageReqVO.getDeptId()); // 包括自身

            pageReqVO.setDeptIds(deptIds);
        }

        IPage<UsersExtRespVO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        IPage<UsersExtRespVO> page = usersExtMapper.selectFacePage(mpPage, pageReqVO);

        return new PageResult<>(page.getRecords(), page.getTotal());
    }

    @Override
    @Transactional(rollbackFor = Exception.class) // 添加事务，异常则回滚所有导入
    public UserImportRespVO importUserList(List<UserExtImportVO> importUsers, Boolean isUpdateSupport) {

        if (CollUtil.isEmpty(importUsers)) {
            throw exception(USER_IMPORT_LIST_IS_EMPTY);
        }

        UserImportRespVO respVO = UserImportRespVO.builder().createUsernames(new ArrayList<>()).updateUsernames(new ArrayList<>()).failureUsernames(new LinkedHashMap<>()).build();

        Map<Long, AdminUserDO> userDOMap = userService.getUserMap(convertList(importUsers, UserExtImportVO::getUserId));

        importUsers.forEach(importUser -> {

            try {
                //获得 更新信息
                UsersExtDO updateDO = BeanUtils.toBean(importUser, UsersExtDO.class);
                //设备部门编号
                updateDO.setDeptId(userDOMap.get(updateDO.getUserId()).getDeptId());

                String url = null;
                String name = null;
                String fileName = importUser.getFaceFile().getOriginalFilename();
                if (fileName == null || fileName.isEmpty()) {

                    name = importUser.getUserId() + "_faceImg.jpg";
                }else {

                    name = importUser.getUserId() + "_faceImg." + fileName.split("\\.")[1];
                }
                byte[] content = IoUtil.readBytes(importUser.getFaceFile().getInputStream());

                //校验 当前用户人脸信息是否已存在
                UsersExtDO usersExtDO = usersExtMapper.selectOne(UsersExtDO::getUserId, importUser.getUserId());
                if (usersExtDO == null) {

                    // 同步 把人脸图片插入 BusinessFile中 存储文件
                    url = fileApi.createBusinessFile(6L, name, content);

                    updateDO.setFaceImg(url);
                    usersExtMapper.insert(updateDO);

                    respVO.getCreateUsernames().add(importUser.getUserName());
                }else {

                    if (!isUpdateSupport) {

                        respVO.getFailureUsernames().put(importUser.getUserName(), USERS_FACE_EXISTS.getMsg());
                    }else {

                        // 变更business_file的content字段内容
                        url = fileApi.updateBusinessFileContent(usersExtDO.getFaceImg(), 6L, name, content);

                        //更新人脸图片
                        updateDO.setId(usersExtDO.getId());
                        updateDO.setFaceImg(url);
                        usersExtMapper.updateById(updateDO);

                        respVO.getUpdateUsernames().add(importUser.getUserName());
                    }
                }

            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        });

        return respVO;
    }
}